/* GtkTetcolor
 * Copyright (C) 1999, 2000 Andrey V. Panov
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef USE_GNOME
# include <gnome.h>
#else
# include <gtk/gtk.h>
#endif
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <sys/stat.h>
#include <gdk/gdkkeysyms.h>

#include "interface.h"
#include "main.h"
#include "callbacks.h"
#include "score.h"
#include "game.h"
#include "preferences.h"
#include "pixmaps.h"

gint stop;
gint cell_width = 28, initial_level = 1, max_score = 0, use_graykeys = 0, destroy_delay = 50;
#ifdef USE_GNOME
gboolean sound_on = 0;
#endif
gint nav_keys[4], alt_nav_keys[4];
GdkFont *font;
gchar *font_name;

GdkPixmap *colors[NUMBER_COLORS + 1], *border_w_gdkxpm, *border_h_gdkxpm;
GdkBitmap *cell_mask[NUMBER_COLORS + 1], *border_w_mask, *border_h_mask;
gchar *color[NUMBER_COLORS + 1][MAX_CELL_SIZE + 3];
gchar *border_width_xpm[2 + border];
gchar *border_height_xpm[2 + MAX_CELL_SIZE * Y_SIZE];
gchar *label_name[MAX_LABEL];


int
main (int argc, char *argv[])
{
  GdkPixmap *icon_xpm;
  GdkBitmap *mask;
  GtkStyle *style;
#include "gtktetcolor.xpm"

#ifdef USE_GNOME
  gnome_score_init ("gtktetcolor");
#endif

#ifdef ENABLE_NLS
  bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
  textdomain (PACKAGE);
#endif

#ifdef USE_GNOME
  gnome_init ("gtktetcolor", VERSION, argc, argv);
#else
  gtk_set_locale ();
  gtk_init (&argc, &argv);
#endif

#ifndef USE_GNOME
  choose_score_file ();
#endif
  load_preferences ();

  main_window = create_main_window ();
  gtk_widget_show (main_window);

  if (use_graykeys)
    {
      nav_keys[0] = GDK_Right;
      nav_keys[1] = GDK_Left;
      nav_keys[2] = GDK_Up;
      nav_keys[3] = GDK_Down;
      alt_nav_keys[0] = GDK_Right;
      alt_nav_keys[1] = GDK_Left;
      alt_nav_keys[2] = GDK_Up;
      alt_nav_keys[3] = GDK_Down;
    }
  else
    {
      nav_keys[0] = GDK_KP_6;
      nav_keys[1] = GDK_KP_4;
      nav_keys[2] = GDK_KP_8;
      nav_keys[3] = GDK_KP_5;
      alt_nav_keys[0] = GDK_KP_Right;
      alt_nav_keys[1] = GDK_KP_Left;
      alt_nav_keys[2] = GDK_KP_Up;
      alt_nav_keys[3] = GDK_KP_Begin;
    }
  create_pixmaps (drawingarea, 1);
  style = gtk_widget_get_style (main_window);
  icon_xpm = gdk_pixmap_create_from_xpm_d (main_window->window, &mask,
					   &style->bg[GTK_STATE_NORMAL],
					   (gchar **) & gtktetcolor_xpm);

  gdk_window_set_icon (main_window->window, NULL, icon_xpm, NULL);
  srand (time (0));

  init_game (main_window);

  gtk_main ();
  return 0;
}

void before_exit (void)
{
  gint i;

  for (i = 0; i < NUMBER_COLORS + 1; i++)
    {
      if (colors[i])
	gdk_pixmap_unref (colors[i]);
      if (cell_mask[i])
	gdk_bitmap_unref (cell_mask[i]);
    }
  if (border_w_gdkxpm)
    gdk_pixmap_unref (border_w_gdkxpm);
  if (border_w_mask)
    gdk_bitmap_unref (border_w_mask);
  if (border_h_gdkxpm)
    gdk_pixmap_unref (border_h_gdkxpm);
  if (border_h_mask)
    gdk_bitmap_unref (border_h_mask);
}
