/* GtkTetcolor
 * Copyright (C) 1999 - 2001 Andrey V. Panov
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef USE_GNOME
# include <gnome.h>
#else
# include <gtk/gtk.h>
#endif
#include <stdlib.h>
#include <gdk/gdkkeysyms.h>
#include <stdio.h>

#include "callbacks.h"
#include "interface.h"
#include "main.h"
#include "score.h"
#include "game.h"
#include "preferences.h"
#include "pixmaps.h"

gint cell_width, left_margin, initial_level, use_graykeys, destroy_delay;
#ifdef USE_GNOME
gboolean sound_on;
#endif
gint nav_keys[4], alt_nav_keys[4];
GtkStyle *label_style;
GdkFont *font;
gchar *font_name;
gchar *label_name[MAX_LABEL];	/* Label identifiers */

gboolean preferences_changed;

void
change_preferences (GtkWidget * widget)
{
  GtkWidget *cell_size_spinbutton, *initial_level_spinbutton, *destroy_delay_spinbutton;
  GtkWidget *fontselection;
  GdkFont *newfont = NULL;
  gint temp;

  cell_size_spinbutton =
    (GtkWidget *) gtk_object_get_data (GTK_OBJECT (widget),
				       "cell_size_spinbutton");
  initial_level_spinbutton =
    (GtkWidget *) gtk_object_get_data (GTK_OBJECT (widget),
				       "initial_level_spinbutton");
  destroy_delay_spinbutton =
    (GtkWidget *) gtk_object_get_data (GTK_OBJECT (widget),
				       "destroy_delay_spinbutton");
  fontselection =
    (GtkWidget *) gtk_object_get_data (GTK_OBJECT (widget), "fontselection");
  temp =
    gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (cell_size_spinbutton));
  if (temp != cell_width)
    {
      cell_width = temp;
      create_pixmaps (drawingarea, 0);
      redraw_all_cells ();
      preferences_changed = TRUE;
    }
  temp =
    gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
				      (initial_level_spinbutton));
  if (temp != initial_level)
    {
      initial_level = temp;
      preferences_changed = TRUE;
    }
  temp =
    gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
				      (destroy_delay_spinbutton));
  if (temp != destroy_delay)
    {
      destroy_delay = temp;
      preferences_changed = TRUE;
    }
  if (use_graykeys)
    {
      nav_keys[0] = GDK_Right;
      nav_keys[1] = GDK_Left;
      nav_keys[2] = GDK_Up;
      nav_keys[3] = GDK_Down;
      alt_nav_keys[0] = GDK_Right;
      alt_nav_keys[1] = GDK_Left;
      alt_nav_keys[2] = GDK_Up;
      alt_nav_keys[3] = GDK_Down;
    }
  else
    {
      nav_keys[0] = GDK_KP_6;
      nav_keys[1] = GDK_KP_4;
      nav_keys[2] = GDK_KP_8;
      nav_keys[3] = GDK_KP_5;
      alt_nav_keys[0] = GDK_KP_Right;
      alt_nav_keys[1] = GDK_KP_Left;
      alt_nav_keys[2] = GDK_KP_Up;
      alt_nav_keys[3] = GDK_KP_Begin;
    }
  newfont = gtk_font_selection_get_font (GTK_FONT_SELECTION (fontselection));
  if (newfont)
    {
      if (font == NULL || !gdk_font_equal (newfont, font))
	{
	  g_free (font_name);
	  font_name =
	    gtk_font_selection_get_font_name (GTK_FONT_SELECTION
					      (fontselection));
	  if (font_name)
	    {
	      change_font_of_labels (newfont);
	      preferences_changed = TRUE;
	      if (font)
		gdk_font_unref (font);
	      font = newfont;
	      gdk_font_ref (font);
	    }
	}
    }
} /* change_preferences */

void
change_font_of_labels (GdkFont * newfont)
{
  GtkWidget *label;
  GtkStyle *newstyle;
  gint i, width;
  guint max_width = 0;
  gchar *str;

  if (newfont)
    {
/*      label = (GtkWidget *) gtk_object_get_data (GTK_OBJECT (main_window),
						 label_name[0]);
      newstyle = gtk_style_copy (gtk_widget_get_style (label));
      gdk_font_unref (newstyle->font);
      newstyle->font = newfont;
      gdk_font_ref (newstyle->font);
      gtk_widget_set_style (label, newstyle); 
*/
      for (i = 0; i < MAX_LABEL; i++)
	{
	  label = (GtkWidget *) gtk_object_get_data (GTK_OBJECT (main_window),
						     label_name[i]);
          newstyle = gtk_style_copy (gtk_widget_get_style (label));
          gdk_font_unref (newstyle->font);
          newstyle->font = newfont;
          gdk_font_ref (newstyle->font);
	  gtk_widget_set_style (label, newstyle);
	  if (i == 2 || i == 4){
	    gtk_label_get (GTK_LABEL(label), &str);
	    width = gdk_string_width (newfont, str);
	    max_width = max_width > width ? max_width : width;
          }
         gdk_font_unref (newstyle->font);
	}
/*      label = (GtkWidget *) gtk_object_get_data (GTK_OBJECT (main_window),
						 label_name[MAX_LABEL - 1]);*/
      width = gdk_string_width (newfont, _("Bonus"));
      max_width = max_width > width ? max_width : width;
      width = gdk_string_width (newfont, _("Game over"));
      max_width = max_width > width ? max_width : width;
      gtk_widget_set_usize (label, max_width, 0);
    }
} /* change_font_of_labels */


void
load_preferences ()
{
  int temp;
#ifdef USE_GNOME
  temp = gnome_config_get_int ("/gtktetcolor/Options/cell_size");
  if (temp >= 10 && temp <= MAX_CELL_SIZE)
    cell_width = temp;
  temp = gnome_config_get_int ("/gtktetcolor/Options/initial_level");
  if (temp >= 1 && temp <= 9)
    initial_level = temp;
  temp = gnome_config_get_int ("/gtktetcolor/Options/use_graykeys");
  if (temp == 1 || temp == 0)
    use_graykeys = temp;
  temp = gnome_config_get_bool ("/gtktetcolor/Options/sound_on");
  if (temp == 1 || temp == 0)
    sound_on = temp;
  temp = gnome_config_get_int ("/gtktetcolor/Options/destroy_delay");
  if (temp > 0 && temp <= MAX_DESTROY_DELAY)
    destroy_delay = temp;
  font_name = gnome_config_get_string ("/gtktetcolor/Options/font");
  if (font_name){
    font = gdk_font_load (font_name);
    if (font)  gdk_font_ref (font); 
  }
#else
  FILE *rcfile;
  gchar *str;
  int i;
# ifdef HAVE_GETLINE
  size_t n = 0;
# endif
# if defined HAVE_GETDELIM
#  ifndef HAVE_GETLINE
  size_t n = 0;
#  endif
  gchar *str1;
# endif

# if !defined HAVE_GETDELIM
  str = (gchar *) g_malloc (20);
# endif  
  rcfile = fopen (rcfile_path, "r");
  if (rcfile == NULL)
    g_print ("cannot open rc file for reading");
  else
    {
      for (i = 0; !feof (rcfile) && i <= 4; i++)
	{
# if defined HAVE_GETDELIM
          n = 0;
          getdelim (&str1, &n, ' ', rcfile);
	  n = strlen (str1);
	  str = g_strndup (str1, n - 1);
# else
	  fscanf (rcfile, "%s ", str);
# endif
	  if (strcmp (str, "cell_size") == 0)
	    {
	      fscanf (rcfile, "%d ", &temp);
	      if (temp >= 10 && temp <= MAX_CELL_SIZE)
		cell_width = temp;
	    }
	  else if (strcmp (str, "initial_level") == 0)
	    {
	      fscanf (rcfile, "%d ", &temp);
	      if (temp >= 1 && temp <= 9)
		initial_level = temp;
	    }
	  else if (strcmp (str, "use_graykeys") == 0)
	    {
	      fscanf (rcfile, "%d ", &temp);
	      if (temp == 1 || temp == 0)
		use_graykeys = temp;
	    }
	  else if (strcmp (str, "destroy_delay") == 0)
	    {
	      fscanf (rcfile, "%d ", &temp);
	      if (temp >= 0 && temp <= MAX_DESTROY_DELAY)
		destroy_delay = temp;
	    }
	  else if (strcmp (str, "font") == 0)
	    {
	      fscanf (rcfile, "%d ", &temp);
	      if (temp > 0)
		{
# ifdef HAVE_GETLINE
		  getline (&font_name, &n, rcfile);
# else
		  font_name = (gchar *) g_malloc (temp + 1);
		  fscanf (rcfile, "%s", font_name);
# endif
		  if (font_name) {
		    font = gdk_fontset_load (font_name);
		    if (font)  gdk_font_ref (font);
		  }    
		}
	    }
# ifdef HAVE_GETLINE
	  else{
	    n = 0; 
	    getline (&str, &n, rcfile); /* read line until the end */
#  if !defined HAVE_GETDELIM
            if (str) g_free (str);
#  endif
          }
# endif
# if defined HAVE_GETDELIM
          if (str) g_free (str);
          if (str1) free (str1);
# endif  
	}
    }
# if !defined HAVE_GETDELIM
  if (str) g_free (str);
# endif  
#endif
} /* load_preferences */

void
save_preferences (void)
{
#ifndef USE_GNOME
  FILE *rcfile;
#endif

#ifdef USE_GNOME
  gnome_config_set_int ("/gtktetcolor/Options/cell_size", cell_width);
  gnome_config_set_int ("/gtktetcolor/Options/initial_level", initial_level);
  gnome_config_set_int ("/gtktetcolor/Options/use_graykeys", use_graykeys);
  gnome_config_set_bool ("/gtktetcolor/Options/sound_on", sound_on);
  gnome_config_set_int ("/gtktetcolor/Options/destroy_delay", destroy_delay);
  if (font_name)
    gnome_config_set_string ("/gtktetcolor/Options/font", font_name);
  gnome_config_sync ();
#else
  rcfile = fopen (rcfile_path, "w");
  if (rcfile == NULL)
    g_print ("cannot open rc file for writing");
  else
    {
      fprintf (rcfile, "cell_size %d\n", cell_width);
      fprintf (rcfile, "initial_level %d\n", initial_level);
      fprintf (rcfile, "use_graykeys %d\n", use_graykeys);
      fprintf (rcfile, "destroy_delay %d\n", destroy_delay);
      if (font_name)
	fprintf (rcfile, "font %d %s\n", strlen (font_name), font_name);
      fclose (rcfile);
    }
#endif
}
